# PyUI
# Copyright (C) 2001-2002 Sean C. Riley
# 
# This library is free software; you can redistribute it and/or
# modify it under the terms of version 2.1 of the GNU Lesser General Public
# License as published by the Free Software Foundation.
# 
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""Dialogs for the pyui library.
"""

import sys
import string

import pyui
from pyui import locals
from pyui.desktop import getDesktop, getRenderer, getTheme

EVENT_OUTPUT = pyui.desktop.getUserEvent()

class ConsoleOutput:
    def __init__(self):
        self.lines = []
        self.oldout = sys.stdout
        self.olderr = sys.stderr
        
    def write(self, text):
        self.oldout.write("%s" % text)
        text = string.rstrip(text)
        if len(text) < 1:
            return
        text = string.replace(text, "\n", " ")
        text = string.replace(text, "\r", " ")      
        self.lines.append(text)
        getDesktop().postUserEvent(EVENT_OUTPUT)

    def beginCapture(self):
        sys.stdout = self
        sys.stderr = self

    def endCapture(self):
        sys.stdout = self.oldout
        self.stderr = self.olderr

    def getLines(self):
        return self.lines
    
    def clear(self):
        self.lines = []

    def __del__(self):
        self.endCapture()
        
#####################
# edit box with history
#####################
class ConsoleEdit(pyui.widgets.Edit):
    def __init__(self, text, max, execCallback):
        pyui.widgets.Edit.__init__(self, text, max, self.onEnter)
        self.history = []
        self.historyPos = 0
        self.execCallback = execCallback
        self.registerEvent(locals.KEYDOWN, self.onKeyDown)

    def onEnter(self, object):
        self.execCallback(self)
        self.history.append(self.text)
        self.historyPos = len(self.history) - 1
        self.setText("")
        self.setDirty()
        return 1

    def onKeyDown(self, event):
        if not self.hasFocus():
            return 0

        if event.key == locals.K_UP:
            if self.history:
                self.historyPos = (self.historyPos - 1) % len(self.history)
                self.setText(self.history[self.historyPos])
            return 1

        if event.key == locals.K_DOWN:
            if self.history:
                self.historyPos = (self.historyPos + 1) % len(self.history)
                self.setText(self.history[self.historyPos])
            return 1
                
        return pyui.widgets.Edit.onKeyDown(self, event)
    
#####################
# box to display lines of text for console output
# or chat windows...
####################
class LineDisplay(pyui.widgets.Base):
    def __init__(self):
        pyui.widgets.Base.__init__(self)
        #self.bgColor = (0,0,49, 255)
        self.lines = []
        self.displayLines = []
        self.numVisible = self.height / locals.TEXT_HEIGHT
        self.numItems = 0
        self.topItem = 0
        self.rewrap = 0
        self.textWidth = 0
        self.vscroll = pyui.widgets.VScroll()
        self.addChild(self.vscroll)
        self.registerEvent(locals.SCROLLPOS, self.onScrollPos)

    def clear(self):
        self.lines = []
        self.displayLines = []
        self.numVisible = self.height / locals.TEXT_HEIGHT
        self.numItems = 0
        self.topItem = 0
        self.rewrap = 0

    def rewrapAll(self):
        self.displayLines = []
        for (line, color) in self.lines:
            self.wrapLine(line, color)
        numLines = len(self.displayLines)
        self.topItem = numLines - self.numVisible
        self.vscroll.setNumItems(numLines, self.numVisible)
        self.vscroll.scrollToItem(self.topItem)

    def wrapLine(self, line, color):
        """Add a line of text to the display lines list with wrapping."""
        (words, spaces) = self.splitLine(line)

        displayLine = ""
        width = 0
        space = ""
        spaceWidth = 0
        while words:
            word = words.pop(0)
            wordWidth = getRenderer().getTextSize(word)[0]

            if width + spaceWidth + wordWidth <= self.textWidth:
                displayLine = displayLine + space + word
                width += spaceWidth + wordWidth
            else:
                self.addDisplayLine(displayLine, color)
                displayLine = word
                width = wordWidth
            space = spaces.pop(0)
            spaceWidth = getRenderer().getTextSize(space)[0]
                
        if displayLine:
            self.addDisplayLine(displayLine, color)

    def splitLine(self, line):
        """Works like split(), but also returns whitespace between words"""
        words = []
        spaces = []
        nEnd = 0
        while nEnd < len(line):
            nStart = nEnd
            while nEnd < len(line) and not line[nEnd].isspace():
                nEnd += 1
            words.append(line[nStart:nEnd])
                
            nStart = nEnd
            nEnd += 1
            while nEnd < len(line) and line[nEnd].isspace():
                nEnd += 1
            spaces.append(line[nStart:nEnd])

        return (words,spaces)

    def addLine(self, line, color = None):
        """This adds lines to the display. it does text wrapping."""
        if not color:
            color = pyui.colors.white
            
        self.lines.append((line, color))
        self.wrapLine(line, color)
        numLines = len(self.displayLines)
        self.topItem = numLines - self.numVisible
        self.vscroll.setNumItems(numLines, self.numVisible)
        self.vscroll.scrollToItem(self.topItem)

    def addDisplayLine(self, displayLine, color):
        self.displayLines.append((displayLine, color))
        self.setDirty()

    def draw(self, renderer):
        if self.rewrap:
            self.rewrapAll()
            self.rewrap = 0
            
        i = 0
        for (line, color) in self.displayLines:
            if i >= self.topItem and i < (self.topItem + self.numVisible):
                renderer.drawText(line, (self.windowRect[0]+2, self.windowRect[1]+2+((i-self.topItem)*locals.TEXT_HEIGHT)), color )
            i += 1
        self.vscroll.draw(renderer)
        self.clearDirty()
        
    def onScrollPos(self, event):
        if event.id == self.vscroll.id:
            self.topItem = event.pos
            self.setDirty()

    def resize(self, w, h):
        pyui.widgets.Base.resize(self, w,h)
        self.vscroll.resize(10, h)
        self.vscroll.moveto(w-10, 0)
        self.textWidth = self.width - self.vscroll.width
        self.numVisible = self.height / locals.TEXT_HEIGHT
        self.rewrap = 1

    def destroy(self):
        self.vscroll.destroy()
        self.vscroll = None
        pyui.widgets.Base.destroy(self)

####################################
# python console window
class Console(pyui.widgets.Frame):

    def __init__(self, x, y, w, h):
        pyui.widgets.Frame.__init__(self, x, y, w, h, "Console")
        self.setLayout(pyui.layouts.BorderLayoutManager())
        self.output = ConsoleOutput()
        self.locals = {}

        # create gui objects
        self.inputBox = ConsoleEdit("", 80, self.onGo)
        self.goButton = pyui.widgets.Button("Go", self.onGo)
        self.outputBox = LineDisplay()

        self.panel = pyui.widgets.Panel()
        self.panel.setLayout(pyui.layouts.BorderLayoutManager())
        self.panel.addChild(self.inputBox, locals.CENTER)
        self.panel.addChild(self.goButton,locals.EAST)

        self.addChild(self.outputBox, locals.CENTER)
        self.addChild(self.panel, locals.SOUTH)
        self.panel.setWindow(self)
        self.pack()

        self.registerEvent(EVENT_OUTPUT, self.onOutput)

    def onOutput(self, event):
        if not self.output:
            return 1
        lines = self.output.getLines()
        for l in lines:
            self.outputBox.addLine(l)
        self.output.clear()
        self.setDirty()
        return 1

    def onGo(self, object):
        command = self.inputBox.text
        self.output.beginCapture()
        try:
            print ">%s" % command
            exec command in globals(), self.locals
        except:
            print "Exception on command '%s':" % command
            print ">    %s" % sys.exc_value
        self.output.endCapture()
        return 1

    def destroy(self):
        #sys.stdout = None
        self.inputBox = None
        self.goButton = None
        self.outputBox = None
        self.panel = None
        pyui.widgets.Frame.destroy(self)
        self.output = None


##################################
# Dialog
#
##################################
class Dialog(pyui.widgets.Frame):
    def __init__(self, x = -1, y = -1, w = 300, h = 200, title = None):
        # center if position not specified
        if x < 0:
            x = (getDesktop().width - w) / 2
        if y < 0:
            y = (getDesktop().height - h) / 2
        pyui.widgets.Frame.__init__(self, x, y, w, h, title)
        self.modal = -1   # this is set to the return value of the dialog
        self.setShow(1)
        self.setDirty()
        
    def doModal(self):
        self.setShow(1)
        getDesktop().setModal(self)

    def close(self, value = 1):
        #print "closed - " , value
        self.modal = value
        getDesktop().setModal(None)
        self.setShow(0)
        self.loseFocus()
        self.postEvent(locals.DIALOGCLOSED)

    def destroy(self):
        if getDesktop().getModal() == self:
            getDesktop().setModal(None)
        pyui.widgets.Frame.destroy(self)

class StdDialog(Dialog):
    def __init__(self, title, text):

        size = getRenderer().getTextSize(title)
        Dialog.__init__(self, title = title)
        self.setLayout(pyui.layouts.BorderLayoutManager())

        self.textLabel = pyui.widgets.Label(text)
        self.textLabel.setText(text)
        self.buttonPanel = pyui.widgets.Panel()
        self.buttonPanel.setLayout(pyui.layouts.BorderLayoutManager())
        self.okButton = pyui.widgets.Button("OK", self.onOK)
        self.okButton.resize(self.innerWidth/2, self.okButton.height)
        self.cancelButton = pyui.widgets.Button("Cancel", self.onCancel)
        self.cancelButton.resize(self.innerWidth/2, self.cancelButton.height)     
        self.buttonPanel.addChild(self.okButton, locals.WEST)
        self.buttonPanel.addChild(self.cancelButton, locals.EAST)
        self.buttonPanel.pack()
        
        self.addChild(self.textLabel, locals.CENTER)
        self.addChild(self.buttonPanel, locals.SOUTH)

        self.pack()

    def onOK(self, button):
        self.close(1)

    def onCancel(self, button):
        self.close(0)

    def draw(self, renderer):
        #print "drawing!!!"
        return Dialog.draw(self, renderer)

    def destroy(self):
        self.buttonPanel = None
        self.okButton = None
        self.cancelButton = None
        self.textLabel = None
        Dialog.destroy(self)
